"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chunk_resolver_1 = require("../src/chunk-resolver");
const pools_1 = require("../src/pools");
function createResolver(options) {
    return new chunk_resolver_1.ChunkResolver(new pools_1.InMemoryPool(), options);
}
function wait(ms) {
    return new Promise((resolve) => setTimeout(resolve, ms));
}
describe('chcache', () => {
    it('should send resolve callback', () => __awaiter(void 0, void 0, void 0, function* () {
        const ttlMs = 20;
        const maxSize = 1;
        const checkIntervalMs = 10;
        const resolver = createResolver({ ttlMs, maxSize, checkIntervalMs });
        const chunks = [];
        resolver.onResolved(chunk => void chunks.push(chunk));
        resolver.cache('table', [{ key: 'value' }]);
        yield wait(ttlMs * 2);
        resolver.stop();
        expect(chunks.length).toBeGreaterThan(0);
    }));
    it('should resolve all rows', () => __awaiter(void 0, void 0, void 0, function* () {
        var _a;
        const ttlMs = 100;
        const maxSize = 100;
        const checkIntervalMs = 25;
        const resolver = createResolver({ ttlMs, maxSize, checkIntervalMs });
        const chunks = [];
        resolver.onResolved(chunk => void chunks.push(chunk));
        const rows = new Array(maxSize).fill(null).map((_, id) => ({ id }));
        resolver.cache('table', rows);
        yield wait(ttlMs * 2);
        resolver.stop();
        expect(chunks.length).toBeGreaterThan(0);
        expect((_a = chunks[0]) === null || _a === void 0 ? void 0 : _a.getRows().length).toEqual(maxSize);
    }));
    it('should resolve by ttlMs', () => __awaiter(void 0, void 0, void 0, function* () {
        var _b, _c, _d;
        const ttlMs = 100;
        const maxSize = 100;
        const checkIntervalMs = 10;
        const resolver = createResolver({ ttlMs, maxSize, checkIntervalMs });
        const chunks = [];
        resolver.onResolved(chunk => void chunks.push(chunk));
        const rows = new Array(maxSize - 1).fill(null).map((_, id) => ({ id }));
        resolver.cache('table', rows);
        yield wait(ttlMs * 2);
        resolver.stop();
        expect(chunks.length).toBeGreaterThan(0);
        expect((_b = chunks[0]) === null || _b === void 0 ? void 0 : _b.isExpired()).toEqual(true);
        expect((_c = chunks[0]) === null || _c === void 0 ? void 0 : _c.isOverfilled(maxSize)).toEqual(false);
        expect((_d = chunks[0]) === null || _d === void 0 ? void 0 : _d.getRows().length).toBeLessThan(maxSize);
    }));
    it('should resolve by maxSize', () => __awaiter(void 0, void 0, void 0, function* () {
        var _e, _f;
        const ttlMs = 500;
        const maxSize = 100;
        const checkIntervalMs = 200;
        const resolver = createResolver({ ttlMs, maxSize, checkIntervalMs });
        const chunks = [];
        resolver.onResolved(chunk => void chunks.push(chunk));
        const rows = new Array(maxSize).fill(null).map((_, id) => ({ id }));
        resolver.cache('table', rows);
        yield wait(ttlMs * 2);
        resolver.stop();
        expect(chunks.length).toBeGreaterThan(0);
        expect((_e = chunks[0]) === null || _e === void 0 ? void 0 : _e.isOverfilled(maxSize)).toEqual(true);
        expect((_f = chunks[0]) === null || _f === void 0 ? void 0 : _f.getRows().length).toEqual(maxSize);
    }));
    it('should create new chunk', () => __awaiter(void 0, void 0, void 0, function* () {
        const ttlMs = 500;
        const maxSize = 100;
        const checkIntervalMs = 250;
        const resolver = createResolver({ ttlMs, maxSize, checkIntervalMs });
        const chunks = [];
        resolver.onResolved(chunk => void chunks.push(chunk));
        const rows = new Array(maxSize).fill(null).map((_, id) => ({ id }));
        resolver.cache('table', rows);
        resolver.cache('table', rows);
        resolver.cache('table', rows);
        resolver.cache('table', rows);
        yield wait(ttlMs * 2);
        resolver.stop();
        expect(chunks.length).toEqual(4);
    }));
    it('should collect rows into a single chunk', () => __awaiter(void 0, void 0, void 0, function* () {
        const ttlMs = 500;
        const maxSize = 100;
        const checkIntervalMs = 100;
        const resolver = createResolver({ ttlMs, maxSize, checkIntervalMs });
        const chunks = [];
        resolver.onResolved(chunk => void chunks.push(chunk));
        const rows = new Array(maxSize / 4).fill(null).map((_, id) => ({ id }));
        resolver.cache('table', rows);
        resolver.cache('table', rows);
        resolver.cache('table', rows);
        resolver.cache('table', rows);
        yield wait(ttlMs * 2);
        resolver.stop();
        expect(chunks.length).toEqual(1);
        expect(chunks[0].getRows().length).toEqual(maxSize);
    }));
});
